#!/usr/bin/python -d
# -*- coding: utf-8 -*-

import os, sys
from PyQt4 import QtCore, QtGui
from PyQt4.Qt import Qt

from ui.select_files_ui import Ui_SelectFilesForm


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

DEFAULT_DIRS = ['Descargas', 'Documentos', 'Escritorio']

class SelectFilesWindow(QtGui.QWidget):
    # Señal que se lanza cuando se finaliza la selección de archivos
    selected_list = QtCore.pyqtSignal(list)
    
    def __init__(self, parent=None):
        super(SelectFilesWindow, self).__init__(parent)
        
        self.ui = Ui_SelectFilesForm()
        self.ui.setupUi(self)
        
        QtCore.QObject.connect(self.ui.acceptButton, QtCore.SIGNAL("clicked(bool)"), self.accept)
        self.selected_list.connect(self.recv_end_signal)
        
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)
        
    def set_path(self, path='./'):
        self.path = path
        folderIcon = QtGui.QIcon.fromTheme("folder")
        documentIcon = QtGui.QIcon.fromTheme("document")
        
        fileList = os.listdir(path)
        
        dirs = []
        files = []
        self.model = QtGui.QStandardItemModel()
        for fileName in fileList:
            if fileName[0] != '.':
                if os.path.isdir(os.path.join(path, fileName)):
                    dirs.append(_fromUtf8(fileName))
                else:
                    files.append(_fromUtf8(fileName))
        dirs = sorted(dirs)
        files = sorted(files)
        
        for dirName in dirs:
            item = QtGui.QStandardItem(dirName)
            
            status = QtCore.QVariant(not Qt.Checked)
            if folderIcon != None:
                item.setIcon(folderIcon)
            if dirName in DEFAULT_DIRS:
                status = QtCore.QVariant(Qt.Checked)
            
            item.setFlags(Qt.ItemIsUserCheckable | Qt.ItemIsEnabled)
            item.setData(status, Qt.CheckStateRole)
            self.model.appendRow(item)
            
        for fileName in files:
            item = QtGui.QStandardItem(fileName)
            
            if documentIcon != None:
                item.setIcon(documentIcon)
            
            item.setFlags(Qt.ItemIsUserCheckable | Qt.ItemIsEnabled)
            item.setData(not QtCore.QVariant(Qt.Checked), Qt.CheckStateRole)
            self.model.appendRow(item)
        
        self.ui.listView.setModel(self.model)
        
    def accept(self):
        items = self.model.takeColumn(0)
        selectedItems = []
        for item in items:
            if item.checkState() == QtCore.QVariant(Qt.Checked):
                selectedItems.append(os.path.join(self.path, "%s" % item.text()))
        self.selected_list.emit(selectedItems)
    
    def recv_end_signal(self, selected):
        self.hide()
            

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    
    app_gui = SelectFilesWindow()
    app_gui.set_path('/home/usuario')
    app_gui.show()
    
    sys.exit(app.exec_())
