#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import shutil
import logging
import signal
import optparse
import traceback

from PyQt4 import QtGui, QtCore
from threading import Thread, Event
from dbus.mainloop.qt import DBusQtMainLoop

from pillbug_client import *
from UI_mig.select_files_window import SelectFilesWindow
from pybytes import bytes2human


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

def configure_logging(debug=False):
    import logging.config
    user_log_path = os.path.join( os.path.expanduser("~"), ".cga")
    user_log_file = "ticbox_applet.log"
    if (not os.path.exists(user_log_path)):
        os.mkdir(user_log_path)
    log_file_name = os.path.join(user_log_path, user_log_file)

    if debug:
        level = 'DEBUG'
        handlers = ['default', 'tofile']
    else:
        level = 'INFO'
        handlers = ['tofile']
        
    log_dict_config = {
                          'version': 1,
                          'formatters': {
                              'custom': {
                                  'format': '%(levelname)s %(asctime)-15s %(module)s %(filename)s:%(lineno)d - %(message)s'
                              },
                          },
                          'handlers': {
                              'default': {
                                  'level': 'DEBUG',
                                  'class': 'logging.StreamHandler',
                                  'formatter': 'custom'
                              },
                              'tofile': {
                                  'level': level,
                                  'class': 'logging.handlers.RotatingFileHandler',
                                  'filename': log_file_name,
                                  'maxBytes': 1048576,
                                  'backupCount': 5,
                                  'formatter': 'custom'
                              },
                           },
                           'loggers': {
                               '': {
                                   'level': level,
                                   'handlers': handlers,
                                   'propagate': True
                               }
                           }
                        }
    logging.config.dictConfig(log_dict_config)


class SysTrayIcon(QtGui.QWidget):
    request_user_quota_signal = QtCore.pyqtSignal()
    request_initial_sync_signal = QtCore.pyqtSignal()
    request_sync_signal = QtCore.pyqtSignal()
    request_set_user_migration = QtCore.pyqtSignal()
    
    animate_icon_signal = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        super(SysTrayIcon, self).__init__(parent)

        self.dbus = PillBugClient()
        self.dbus.signal_registers(self.request_user_quota_signal,
                                   self.request_initial_sync_signal,
                                   self.request_sync_signal,
                                   self.request_set_user_migration)
        
        self.notify_msg = lambda title: NotifyMsg(title)
        self.notify_error = lambda title, msg: NotifyMsg(title, msg, dialog="dialog-error")
        self.register_handlers()
        self.path = os.path.join(self.dbus.get_home_dir(), 'Documentos')

        self.quota_total = 0.0
        self.quota_used = 0.0
        
        self.initial_sync_done = False
        self.last_progress = 100
        self.animate_dir = True
        self.animate_index = 1

        self.createSystrayIcon()
        self.initial_rsync()
        self.systrayIcon.show()

    def createSystrayIcon(self):
        self.show_quota_action = QtGui.QAction("--",     self, triggered=self.updateUI)
        self.show_quota_action.setVisible(False)
        self.update_action     = QtGui.QAction("Actualizar ahora",     self, triggered=self.update_rsync)
        self.open_dir_action   = QtGui.QAction("Abrir carpeta Documentos",     self, triggered=self.open_dir)
        
        self.systrayIconMenu = QtGui.QMenu(self)
        
        self.systrayIconMenu.addAction(self.show_quota_action)
        self.systrayIconMenu.addSeparator()
        self.systrayIconMenu.addAction(self.open_dir_action)
        self.systrayIconMenu.addSeparator()
        self.systrayIconMenu.addAction(self.update_action)
        
        self.systrayIcon = QtGui.QSystemTrayIcon(self)
        self.systrayIcon.setContextMenu(self.systrayIconMenu)
        self.systrayIcon.setIcon(QtGui.QIcon("icons/waiting/iconw1.png"))
        self.systrayIcon.setVisible(True)

    def register_handlers(self):
        self.animate_icon_signal.connect(self.animate_icon)
        self.dbus.sync_started_signal.connect(self.show_syncro_status)
        self.dbus.quota_data_signal.connect(self.handler_quota)
        self.dbus.status_signal.connect(self.handler_status)
        self.dbus.progress_signal.connect(self.handler_progress)

    def handler_quota(self, used, total):
        logging.debug("Quota recibida (Kbytes): %s/%s" % (total, used))
        self.quota_total = float(total) #in Kbytes
        self.quota_used = float(used)
        self.updateUI()

    def handler_status(self, status):
        if status == 0: #Todo acabó bien...
            logging.debug("Status recibido: %d" % status)
            self.update_action.setText("Sincronizar ahora")
            self.update_action.setEnabled(True)
            self.animation.stop()
            self.animate_dir = True
            self.animate_index = 1
            
            logging.info("Sincronización terminada con éxito.")
            if self.dbus.is_movil_user() and not self.initial_sync_done:
                self.mig_window = SelectFilesWindow()
                self.mig_window.set_path(self.path)
                self.mig_window.show()
                self.mig_window.selected_list.connect(self.user_migration)
                self.initial_sync_done = True
                    
            self.notify_msg(title="Sincronización completada")
            
            self.request_user_quota_signal.emit()
        else: #Hay algún error...
            logging.error("Status recibido: %d" % status)
            self.show_error_status(status)

    def user_migration(self, selected):
        logging.info("Migrando (%s)" % self.path)
        fileList = os.listdir(self.path)
        for name in fileList:
            fullName = os.path.join(self.path, name)
            if not (fullName in selected):
                if os.path.isdir(fullName):
                    shutil.rmtree(fullName)
                else:
                    os.remove(fullName)
        self.update_rsync()
        self.request_set_user_migration.emit()

    def initial_rsync(self):
        logging.info("Sincronización inicial")
        self.notify_msg(title="Iniciando sincronización")
        self.request_initial_sync_signal.emit()

    def update_rsync(self):
        self.notify_msg(title="Iniciando sincronización")
        self.request_sync_signal.emit()
    
    def show_syncro_status(self):
        self.update_action.setDisabled(True)
        self.update_action.setText("Sincronizando...")
        self.animation = PeriodicSignal(0.12, self.animate_icon_signal)
        self.animation.start()
    
    def handler_progress(self, done, total):
        self.update_action.setText("Sincronizando archivo %d de %d" % (done, total))
    
    def show_error_status(self, error_code=1):
        self.update_action.setText("Sincronizar ahora")
        self.update_action.setEnabled(True)
        self.animation.stop()
        self.systrayIcon.setIcon(QtGui.QIcon("icons/icon_11.png"))
        
        if not self.show_quota_action.isVisible():
                self.show_quota_action.setVisible(True)
        
        if error_code == 2:
            self.show_quota_action.setText("Error: necesita liberar espacio")
            self.notify_error(title="No se han sincronizado todos los archivos", msg="Necesita liberar espacio")
        else:
            self.show_quota_action.setText("Error en la conexión")
            self.notify_error(title="Error en la sincronización", msg="Error conectando con el servidor")
        
        
    def animate_icon(self):
        if self.animate_index < 10:
            self.animate_index = self.animate_index + 1
        else:
            self.animate_index = 1
        
        self.systrayIcon.setIcon(QtGui.QIcon("icons/waiting/iconw%d.png" % self.animate_index))

    def updateUI(self):
        if self.quota_total > 0:
            porc_usado = (self.quota_used * 100) / self.quota_total
            total = bytes2human(self.quota_total*1024)
            usado = bytes2human(self.quota_used*1024)

            self.show_quota_action.setText("Usado %.0f%% (%s de %s)" % (porc_usado, usado, total))
            if not self.show_quota_action.isVisible():
                self.show_quota_action.setVisible(True)

            #El Icono...
            if porc_usado > 90 or self.last_progress < 100: # (last_progress < 100) => Sincronización incompleta!
                icon_index = 11
            else:
                icon_index = int(round(porc_usado/10, 0))
                if icon_index == 0:
                        icon_index = 1
            self.systrayIcon.setIcon(QtGui.QIcon("icons/icon_%d.png" % icon_index))
        else:
            self.show_error_status()
            
    def open_dir(self):
        os.system("nautilus %s " % self.path)


class NotifyMsg(object):
        def __init__(self, title="", msg="", dialog="dialog-info", parent=None):
                import pynotify
                pynotify.init('app')
                n = pynotify.Notification(title, msg, dialog) # Titulo, Mensaje...
                n.show()

              
if __name__ == '__main__':
    parser = optparse.OptionParser()
    parser.add_option('--debug', help='start in debug mode',
                      dest="debug", action="store_true", default=False)
    (options, args) = parser.parse_args()
    
    configure_logging(options.debug)
    
    try:
        DBusQtMainLoop(set_as_default=True)
        app = QtGui.QApplication([])
        signal.signal(signal.SIGINT, signal.SIG_DFL)
        systrayicon = SysTrayIcon()
        app.exec_()
    except Exception:
        logging.critical("Excepción:\n %s" % traceback.format_exc())
