# -*- coding: UTF-8 -*-

import ldap
import getpass
import socket
import sys

import logging
logging.basicConfig(level=logging.DEBUG,
                    format="%(asctime)s %(filename)s:%(lineno)d - %(levelname)s: %(message)s")

class LdapUser:
    
    def __init__(self, uid):
        self.uid = uid
        base = self.get_base()
        baseDN = "uid=%s,ou=People,%s" % (self.uid, base)
        
        
        lcon = ldap.open("c0")
        search_id = lcon.search(baseDN, ldap.SCOPE_SUBTREE)
        result_type, result_data = lcon.result(search_id, 0)

        self.result_data = result_data

        for f_key, f_val in self.result_data:
            for key, val in f_val.items():
                if key == 'homeDirectory':
                    self.homeDirectory = val[0]

    def is_movil(self):
        return self.homeDirectory.startswith('/home/nfs/')
    
    def get_home_dir(self):
        return self.homeDirectory

    def get_base(self):
        ldap_conf = open('/etc/ldap.conf').readlines()
        if len(ldap_conf) > 0:
            ldap_conf.reverse()
            for line in ldap_conf:
                if line.startswith('base '):
                    return line.replace('base ', '')
        return ''
    
    def set_user_migration(self):
        
        HOST = 'f0'
        PORT = 30000
        
        loged_user = getpass.getuser()
        
        # comprobamos si hay conectividad con el equipo
        if( self.__test_connection(HOST, PORT)==True):
            logging.debug("hay conexión con %s:%s" %(HOST, PORT))
            result = self.__send_command(loged_user, HOST, PORT)
            return True
        else:
            logging.info("imposible conectar con %s por el puerto %s"
                             %(HOST, PORT))
            return False
            
    def __test_connection(self, target_host, port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(0.5)
        result = -1
        #connect_ex no da excepciones, devuelve el valor 
        result = s.connect_ex((target_host, port))
        logging.debug("Socket.Connect returns code %s" %result)
        s.close()
        return result==0

    def __send_command(self, command, host, port):
        res    = 1
        tries = 3
        while ( tries > 0 ):
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect((host, port))
                logging.debug("Sent from %s" %( repr(command)))
                returned = s.send(command)
                #logging.debug("The call send returns %s (bytes sent?)" %repr(returned))
                res = 0
                break
            except Exception as e:
                logging.error( "Error sending command %s to %s:%s \nReason: %s"
                                    %(command, host, port, str(e)))
            finally:
                s.close()
                tries -= 1
        if( tries==0 and res!=0):
            logging.error("After 3 tries the request can't be sent %s:%s" %(host,port))
        return res