#!/usr/bin/python
# -*- coding:utf-8 -*-

import dbus
import dbus.service
import logging
import time

from dbus.mainloop.glib import DBusGMainLoop

from pillbug_exceptions import *
from synchronizer import Synchronizer


class PillBug_daemon(dbus.service.Object):

    def __init__(self):
        logging.info("Inicializando interfaz en dbus: SystemBus")
        self.bus = dbus.SystemBus()
        bus_name = dbus.service.BusName('org.cga.ticbox.pillbug', self.bus)
        dbus.service.Object.__init__( self,
                                      bus_name,
                                      '/org/cga/ticbox/pillbug')
        
        self.bus.add_signal_receiver(self.initial_sync_data,
                                     'initial_sync_data', 
                                     'org.cga.ticbox.pillbug')
        self.bus.add_signal_receiver(self.sync_data,
                                     'sync_data', 
                                     'org.cga.ticbox.pillbug')
        
        self.flag_file = '/tmp/.pillbug'


    ## Este método devuelve la información de quota perteneciente al usuario
    # con nombre, user id y group id dados.
    @dbus.service.method('org.cga.ticbox.pillbug',
                         signature='suus',out_signature='au')
    def get_user_data(self, name, uid, guid, home_dir):
        logging.debug("Se ejecuta get_user_data con argumentos: %s, %s, %s"
                      %(name, uid, guid))
        
        s = Synchronizer(name, uid, guid, home_dir)
        
        result = s.get_quota_info()
        logging.debug("Synchronizer devuelve los datos para %s(%s:%s) %s"
                      %(name, uid, guid, result))
        return result
    
    
    @dbus.service.signal('org.cga.ticbox.pillbug',
                         signature='uu')
    def sync_status(self, uid, status):
        logging.debug("Emitiendo señal de estado con %s" %status)
    
    
    def initial_sync_data(self, name, uid, guid, home_dir):
        logging.info("Se solicita sincronizacion INICIAL por %s (%s/%s)"
                      %(name, uid, guid))
        self.__sync(name, uid, guid, home_dir, initial=True)
    
    
    def sync_data(self, name, uid, guid, home_dir):
        logging.info("Se solicita sincronizacion de %s (%s/%s)"
                      %(name, uid, guid))
        self.__sync(name, uid, guid, home_dir)
    
    
    def __sync(self, name, uid, guid, home_dir, initial=False):
        self.set_flag()
        try:
            s = Synchronizer(name, uid, guid, home_dir)
            
            if not initial:
                s.rsync_from_local(delete=True,
                                   progress_handler=self.sync_progress)
            else:
                s.rsync_from_nfs(progress_handler=self.sync_progress)
            
            self.sync_status(uid, 0)
        
        except QuotaExceeded:
            self.sync_status(uid, 2)
        except Exception as e:
            logging.error("Exception!! : %s" %e)
            self.sync_status(uid, 1)
        finally:
            self.unset_flag()
            pass
        
        
    def set_flag(self):
        flag = open(self.flag_file, 'w')
        flag.write("%s" % time.time())
        flag.close()
        pass
    
    
    def unset_flag(self):
        flag = open(self.flag_file, 'w')
        flag.close()
        pass
    
    
    @dbus.service.signal('org.cga.ticbox.pillbug',
                         signature='uuu')
    def sync_progress(self, uid, done, total):
        logging.debug("Emitida señal de sincronizando a %s, en proceso %s de %s archivos" 
                      %(uid, done, total))
        
        