#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import gobject
import os, os.path
import sys
import re
import subprocess
import logging

from pillbug_exceptions import *


class Synchronizer:
    
    def __init__(self, user_name, user_id, user_guid, home_dir):
        #TODO: ¿ es necesario comprobar que uids y demás son validos?
        self.__user_name = str(user_name)
        self.__user_id = user_id
        self.__user_guid = user_guid
        
        self.nfs_server = "c0"
        self.server_path = "/home/nfs/"+self.__user_name
        # es importante la barra del punto de montaje para que a la hora de
        # sincronizar sincronice el contenido de la carpeta fuente en ~/Documentos
        self.mount_point = os.path.join("/var/cache/nfs/", self.__user_name+"/")
        self.ticbox_path = os.path.join(home_dir, 'Documentos/')
    
    def mount(self):
        
        if not os.path.exists(self.mount_point):
            logging.info("No hay punto de montaje, se crea")
            os.mkdir(self.mount_point)
        
        if os.path.ismount(self.mount_point):
            self.umount(self.mount_point)
        
        # si llegados aquí todo va bien, listo para montar
        command = ["mount","-t","nfs",
                   self.nfs_server+":"+self.server_path,
                   self.mount_point]
        
        (err_code, output) =self.execute_command(command)
        if(err_code >0):
            logging.error( "Error montando: %s / %s" %(err_code, command))
            return False
        else:
            logging.debug( "Sistema de ficheros nfs montado en %s" 
                           %(self.mount_point))
            return True
        
    
    
    def umount(self, path):
        command = ["umount",path]
        
        (err_code, output) = self.execute_command(command)
        
        if(err_code ==130):
            logging.error( "Nada montado en %s" %(err_code))
        elif(err_code==0):
            logging.info( "Sistema de ficheros nfs desmontado en "+ path)
        elif (err_code>0):
            logging.error( "Error desmontando: %s" %(err_code))
        
    
    def execute_command_as_user( self, command_array):
        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        logging.debug("Me hago usuario %s con su uid(%s) y gid(%s)"
                        %(self.__user_name,
                          self.__user_id,
                          self.__user_guid))
        result = self.execute_command(command_array)
        os.seteuid(0)
        os.setegid(0)
        logging.debug("Recupero mi uid(%s) y gid(%s)"
                      %(os.getuid(),
                        os.getgid()))
        return result
    
    
    def execute_command(self, command_array):
        # este será el código devuelto por DEFAAAUUULT (con voz de chiquito)
        return_code = 0
        try:
            output = subprocess.check_output(command_array)
        except subprocess.CalledProcessError as error:
            logging.error("Error ejecutando comando (error_code: %d)" % error.returncode)
            return_code = error.returncode 
            output = error.output
        return (return_code, output)
    
    
    def check_ticbox(self):
        def f():
            logging.debug("Comprobando directorio %s con uid(%s) gid(%s)"
                          %(self.ticbox_path,
                            self.__user_id,
                            self.__user_guid))
            if not os.path.isdir(self.ticbox_path):
                os.mkdir(self.ticbox_path)
        self.run_as_user(f)
        
    def get_user_ticbox(self, username):
        def f():
            return os.path.join(os.path.expanduser("~"),"Documentos") 
        return self.run_as_user(f)
        
        
    def run_as_user(self, function):
        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        result = function()
        os.seteuid(0)
        os.setegid(0)
        return result
    
    #
    #    SYNCHRO    STUFFS
    #
    def rsync_from_nfs(self,progress_handler=None):
        self.__rsync(source=self.mount_point, 
                     dest=self.ticbox_path, 
                     progress_handler=progress_handler)
        
    ## Este metodo realizará una sincronizacion con
    #     origen:  carpeta TicBox local
    #     destino: punto de montaje nfs
    #     opcion --delete: ACTIVA
    def rsync_from_local(self, delete=False, progress_handler=None):
        self.__rsync( source=self.ticbox_path, 
                      dest=self.mount_point, 
                      delete=delete,
                      progress_handler=progress_handler)
        
    
    ## Este metodo realizará una sincronizacion con
    #     origen:  punto de montaje nfs
    #     destino: carpeta TicBox local
    #     opcion --delete: DESACTIVADA
    def __rsync(self, source, dest, delete=False, progress_handler=None):
        rsync_command = ["rsync", "-avur", source, dest]
        if delete:
            rsync_command.append("--delete")
        
        #FIXME: sería mejor hacerlo en el constructor? Qué pasa si multisesión?
        self.check_ticbox()
        
        self.mount()
        logging.debug("Iniciando sincronizacion entre %s y %s" 
                      %(source, dest))
        if not callable(progress_handler):
            (return_code, output) = self.execute_command_as_user(rsync_command)
            logging.debug("El rsync dice \"%s\", codigo devuelto %s" 
                          %(output, return_code))
        else:
            os.setegid(self.__user_guid)
            os.seteuid(self.__user_id)

            to_synch_array = self.__get_total_files_to_sync(rsync_command)
            total_files = len(to_synch_array)
            
            process = subprocess.Popen( rsync_command, 
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE)
            # marca cuando hayamos encontrado la primera línea
            flag = False
            files_synchronized = []
            while True:
                line = process.stdout.readline().strip()
                if not flag and line == 'sending incremental file list':
                    flag = True
                elif flag and not line.endswith('/') and not line.startswith('/') and total_files != 0 and len(files_synchronized) < total_files:
                    files_synchronized.append(object)
                    logging.debug("sincronizado %s" %(line))
                    # llamamos a la señal con: uid, sincronizado, total a sincronizar
                    progress_handler(self.__user_id, 
                                     len(files_synchronized),
                                     total_files)
                return_code = process.poll()
                if return_code != None:
                    break
                    
            os.seteuid(0)
            os.setegid(0)
            logging.debug(" HARD DEBUG: return_code = %s " % return_code)
        
        self.umount(self.mount_point)
        if return_code == 12:
            raise QuotaExceeded
        elif return_code > 0:
            logging.info("*** rsync ha terminado con return_code = %s    ¿ERROR?" % return_code)
    
    
    def __get_total_files_to_sync(self, in_command_array):
        #cambiamos las opciones para que se ejecute con -n, dry run
        command_array = list(in_command_array)
        command_array[1] = '-navur'
        process = subprocess.Popen( command_array, 
                                     stdout=subprocess.PIPE)
    
        # esperamos al hijo para así contar todo
        process.wait()
        
        output = process.stdout.readlines()
        files_to_sync = []
        guia = False
        for element in output:
            element = element.strip()
            if element == 'sending incremental file list':
                guia = True
                pass
            elif len(element) == 0:
                break
            elif  guia and not element.endswith('/') and not element.startswith('/'):
                logging.debug("*** ELEM: %s " % element)
                files_to_sync.append(element)
        logging.debug("Total de archivos a sync: %s" %len(files_to_sync))
        return files_to_sync

    
    ## Dado un nombre de usuario devuelve un array con la siguiente informacion
    # referente a la quota de nfs:
    #  - quota usada 
    #  - limite de quota
    def get_quota_info(self):
        self.mount()
        quota_command = ["quota", "-wp", "-u", self.__user_name]
        (return_code, output) = self.execute_command(quota_command)
        if return_code>0:
            logging.error("Error consultando la quota")
            return [0, 0]
        
        # cogemos solo la tercera linea de la salida y nos quedamos con lo que
        # no sean espacios, algo así:
        # ['c0:/home/nfs/ssoto', '15548', '102400', '102400', '0', '6', '0', '0', '0']
        cute_output = None
        for line in output.split('\n'):
            if line.startswith('c0'):
                cute_output = line.split()
                break
        
        self.umount(self.mount_point)
        
        if cute_output is None:
            return [0, 0] 
        
        used   =   int(cute_output[1]) 
        limit = int(cute_output[2])
        logging.debug("la informacion de %s(%s:%s) es: used==>%s limit==>%s"
                      %(self.__user_name,self.__user_id,self.__user_guid,
                        used,limit))
        return [used, limit]
    
